/** @file sp/spBase.h
 */

#ifndef __SPBASE_H
#define __SPBASE_H

#include <sp/spDefs.h>
#include <sp/spMemory.h>
#include <sp/spString.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spBaseGroup Basic Functions */

/** @addtogroup spBaseGroup
 *  @{ */  /*----@addtogroup spBaseGroup----*/

typedef void (*spPrintFunc)(const char *message);

/** @~english Indicates the file kind for applications to deal with documents. */
typedef enum {
    SP_FILE_KIND_UNKNOWN = -1,     /**< @~english Unknown files. */
    SP_FILE_KIND_DOCUMENT = 0,     /**< @~english General documents. */
    SP_FILE_KIND_SOURCE = 1,       /**< @~english Source files. */
    SP_FILE_KIND_CONFIG = 2,       /**< @~english Configure files. */
    SP_FILE_KIND_INCLUDE = 3,      /**< @~english Include files. */
    SP_FILE_KIND_PICTURE = 10,     /**< @~english Pictures and images. */
    SP_FILE_KIND_FIGURE = 11,      /**< @~english Figures. */
    SP_FILE_KIND_MUSIC = 20,       /**< @~english MIDI files and music files. */
    SP_FILE_KIND_SOUND = 21,       /**< @~english Other sound files. */
    SP_FILE_KIND_MOVIE = 25,       /**< @~english Movie files. */
    SP_FILE_KIND_APPLICATION = 30, /**< @~english Applications. */
    SP_FILE_KIND_LIBRARY = 31,     /**< @~english Library files. */
    SP_FILE_KIND_ARCHIVE = 40,     /**< @~english Archive files. */
    SP_FILE_KIND_URI = 50,         /**< @~english URI links. */
} spFileKind;

typedef unsigned long spPaperOrientation;
#define SP_PAPER_ORIENTATION_HORIZONTAL_MASK (1L<<4)    
#define SP_PAPER_ORIENTATION_REVERSE_MASK (1L<<5)    
#define SP_PAPER_ORIENTATION_UNKNOWN_MASK (1L<<6)
#define SP_PAPER_ORIENTATION_PORTRAIT (0L)
#define SP_PAPER_ORIENTATION_LANDSCAPE SP_PAPER_ORIENTATION_HORIZONTAL_MASK
#define SP_PAPER_ORIENTATION_SEASCAPE (SP_PAPER_ORIENTATION_HORIZONTAL_MASK|SP_PAPER_ORIENTATION_REVERSE_MASK)
#define SP_PAPER_ORIENTATION_UPSIDE_DOWN SP_PAPER_ORIENTATION_REVERSE_MASK
#define SP_PAPER_ORIENTATION_UNKNOWN SP_PAPER_ORIENTATION_UNKNOWN_MASK

#define spIsPaperOrientationHorizontal(orient) (((orient) & SP_PAPER_ORIENTATION_HORIZONTAL_MASK) ? SP_TRUE : SP_FALSE)


#define SP_PAPER_SIZE_STRING_UNIT_BOTH (1L<<0)   /* such as `10 mm x 10 mm' */
#define SP_PAPER_SIZE_STRING_UNIT_SYMBOL (1L<<1) /* `1 x 1"' instead of `1 x 1 in' */
#define SP_PAPER_SIZE_STRING_NO_UNIT (1L<<2)     /* such as `10 x 10' */
#define SP_PAPER_SIZE_STRING_NO_SPACE (1L<<3)    /* such as `10x10mm' */
#define SP_PAPER_SIZE_STRING_INCLUDE_SIZE (1L<<6)
#define SP_PAPER_SIZE_STRING_INCH (1L<<8)
  
typedef unsigned long spPaperSize;
#define SP_PAPER_SIZE_NUMBER_MASK 0xf    
#define SP_PAPER_SIZE_MASK 0xfff    
#define SP_PAPER_TYPE_MASK 0xf000
#define SP_PAPER_SIZE_0_MASK (0L)
#define SP_PAPER_SIZE_1_MASK (1L)
#define SP_PAPER_SIZE_2_MASK (2L)
#define SP_PAPER_SIZE_3_MASK (3L)
#define SP_PAPER_SIZE_4_MASK (4L)
#define SP_PAPER_SIZE_5_MASK (5L)
#define SP_PAPER_SIZE_6_MASK (6L)
#define SP_PAPER_SIZE_7_MASK (7L)
#define SP_PAPER_SIZE_8_MASK (8L)
#define SP_PAPER_SIZE_9_MASK (9L)
#define SP_PAPER_SIZE_10_MASK (10L)
#define SP_PAPER_SIZE_2_0_MASK (11L)
#define SP_PAPER_SIZE_4_0_MASK (12L)
#define SP_PAPER_SIZE_LETTER_MASK (1L<<4)
#define SP_PAPER_SIZE_LEGAL_MASK (1L<<5)
#define SP_PAPER_SIZE_TABLOID_MASK (1L<<6)
#define SP_PAPER_SIZE_HAGAKI_MASK (1L<<7)
#define SP_PAPER_SIZE_STATEMENT_MASK (1L<<8)
#define SP_PAPER_SIZE_QUARTO_MASK (1L<<9)
#define SP_PAPER_SIZE_FOOLSCAP_MASK (1L<<10)
#define SP_PAPER_SIZE_EXECUTIVE_MASK (1L<<11)
#define SP_PAPER_SIZE_POST_MASK (1L<<12)
#define SP_PAPER_SIZE_CROWN_MASK (1L<<13)
#define SP_PAPER_SIZE_ROYAL_MASK (1L<<14)
#define SP_PAPER_SIZE_DEMY_MASK (1L<<15)
#define SP_PAPER_SIZE_ELEPHANT_MASK (1L<<16)
#define SP_PAPER_SIZE_PRINCESS_MASK (1L<<17)
#define SP_PAPER_SIZE_EMPEROR_MASK (1L<<18)
#define SP_PAPER_SIZE_SMALL_MASK (1L<<19)
#define SP_PAPER_SIZE_MEDIUM_MASK (1L<<20)
#define SP_PAPER_SIZE_DOUBLE_MASK (1L<<21)
#define SP_PAPER_SIZE_LARGE_MASK (1L<<22)
#define SP_PAPER_SIZE_CUSTOM_MASK (1L<<23)
#define SP_PAPER_TYPE_A_MASK (1L<<24)
#define SP_PAPER_TYPE_B_MASK (1L<<25)
#define SP_PAPER_TYPE_C_MASK (1L<<26)
#define SP_PAPER_TYPE_JIS_B_MASK (1L<<27)
#define SP_PAPER_TYPE_ANSI_MASK (1L<<28)
#define SP_PAPER_TYPE_ARCH_MASK (1L<<29)
#define SP_PAPER_TYPE_PA_MASK (1L<<30)
#define SP_PAPER_TYPE_US_MASK (1L<<31)
    
#define SP_PAPER_SIZE_UNKNOWN 0L
#define SP_PAPER_SIZE_A0 (SP_PAPER_TYPE_A_MASK | SP_PAPER_SIZE_0_MASK)
#define SP_PAPER_SIZE_A1 (SP_PAPER_TYPE_A_MASK | SP_PAPER_SIZE_1_MASK)
#define SP_PAPER_SIZE_A2 (SP_PAPER_TYPE_A_MASK | SP_PAPER_SIZE_2_MASK)
#define SP_PAPER_SIZE_A3 (SP_PAPER_TYPE_A_MASK | SP_PAPER_SIZE_3_MASK)
#define SP_PAPER_SIZE_A4 (SP_PAPER_TYPE_A_MASK | SP_PAPER_SIZE_4_MASK)
#define SP_PAPER_SIZE_A5 (SP_PAPER_TYPE_A_MASK | SP_PAPER_SIZE_5_MASK)
#define SP_PAPER_SIZE_A6 (SP_PAPER_TYPE_A_MASK | SP_PAPER_SIZE_6_MASK)
#define SP_PAPER_SIZE_2A0 (SP_PAPER_TYPE_A_MASK | SP_PAPER_SIZE_2_0_MASK)
#define SP_PAPER_SIZE_4A0 (SP_PAPER_TYPE_A_MASK | SP_PAPER_SIZE_4_0_MASK)

#define SP_PAPER_SIZE_B0 (SP_PAPER_TYPE_B_MASK | SP_PAPER_SIZE_0_MASK)
#define SP_PAPER_SIZE_B1 (SP_PAPER_TYPE_B_MASK | SP_PAPER_SIZE_1_MASK)
#define SP_PAPER_SIZE_B2 (SP_PAPER_TYPE_B_MASK | SP_PAPER_SIZE_2_MASK)
#define SP_PAPER_SIZE_B3 (SP_PAPER_TYPE_B_MASK | SP_PAPER_SIZE_3_MASK)
#define SP_PAPER_SIZE_B4 (SP_PAPER_TYPE_B_MASK | SP_PAPER_SIZE_4_MASK)
#define SP_PAPER_SIZE_B5 (SP_PAPER_TYPE_B_MASK | SP_PAPER_SIZE_5_MASK)
#define SP_PAPER_SIZE_B6 (SP_PAPER_TYPE_B_MASK | SP_PAPER_SIZE_6_MASK)

#define SP_PAPER_SIZE_JIS_B0 (SP_PAPER_TYPE_JIS_B_MASK | SP_PAPER_SIZE_0_MASK)
#define SP_PAPER_SIZE_JIS_B1 (SP_PAPER_TYPE_JIS_B_MASK | SP_PAPER_SIZE_1_MASK)
#define SP_PAPER_SIZE_JIS_B2 (SP_PAPER_TYPE_JIS_B_MASK | SP_PAPER_SIZE_2_MASK)
#define SP_PAPER_SIZE_JIS_B3 (SP_PAPER_TYPE_JIS_B_MASK | SP_PAPER_SIZE_3_MASK)
#define SP_PAPER_SIZE_JIS_B4 (SP_PAPER_TYPE_JIS_B_MASK | SP_PAPER_SIZE_4_MASK)
#define SP_PAPER_SIZE_JIS_B5 (SP_PAPER_TYPE_JIS_B_MASK | SP_PAPER_SIZE_5_MASK)
#define SP_PAPER_SIZE_JIS_B6 (SP_PAPER_TYPE_JIS_B_MASK | SP_PAPER_SIZE_6_MASK)
    
#define SP_PAPER_SIZE_ANSI_A (SP_PAPER_TYPE_ANSI_MASK | SP_PAPER_SIZE_4_MASK)
#define SP_PAPER_SIZE_ANSI_B (SP_PAPER_TYPE_ANSI_MASK | SP_PAPER_SIZE_3_MASK)
#define SP_PAPER_SIZE_ANSI_C (SP_PAPER_TYPE_ANSI_MASK | SP_PAPER_SIZE_2_MASK)
#define SP_PAPER_SIZE_ANSI_D (SP_PAPER_TYPE_ANSI_MASK | SP_PAPER_SIZE_1_MASK)
#define SP_PAPER_SIZE_ANSI_E (SP_PAPER_TYPE_ANSI_MASK | SP_PAPER_SIZE_0_MASK)

#define SP_PAPER_SIZE_ARCH_A (SP_PAPER_TYPE_ARCH_MASK | SP_PAPER_SIZE_4_MASK)
#define SP_PAPER_SIZE_ARCH_B (SP_PAPER_TYPE_ARCH_MASK | SP_PAPER_SIZE_3_MASK)
#define SP_PAPER_SIZE_ARCH_C (SP_PAPER_TYPE_ARCH_MASK | SP_PAPER_SIZE_2_MASK)
#define SP_PAPER_SIZE_ARCH_D (SP_PAPER_TYPE_ARCH_MASK | SP_PAPER_SIZE_1_MASK)
#define SP_PAPER_SIZE_ARCH_E (SP_PAPER_TYPE_ARCH_MASK | SP_PAPER_SIZE_0_MASK)
#define SP_PAPER_SIZE_ARCH_E1 (SP_PAPER_TYPE_ARCH_MASK | SP_PAPER_SIZE_5_MASK)

#define SP_PAPER_SIZE_PA0 (SP_PAPER_TYPE_PA_MASK | SP_PAPER_SIZE_0_MASK)
#define SP_PAPER_SIZE_PA1 (SP_PAPER_TYPE_PA_MASK | SP_PAPER_SIZE_1_MASK)
#define SP_PAPER_SIZE_PA2 (SP_PAPER_TYPE_PA_MASK | SP_PAPER_SIZE_2_MASK)
#define SP_PAPER_SIZE_PA3 (SP_PAPER_TYPE_PA_MASK | SP_PAPER_SIZE_3_MASK)
#define SP_PAPER_SIZE_PA4 (SP_PAPER_TYPE_PA_MASK | SP_PAPER_SIZE_4_MASK)
#define SP_PAPER_SIZE_PA5 (SP_PAPER_TYPE_PA_MASK | SP_PAPER_SIZE_5_MASK)
#define SP_PAPER_SIZE_PA6 (SP_PAPER_TYPE_PA_MASK | SP_PAPER_SIZE_6_MASK)
    
#define SP_PAPER_SIZE_LETTER SP_PAPER_SIZE_LETTER_MASK
#define SP_PAPER_SIZE_GOVERNMENT_LETTER (SP_PAPER_TYPE_US_MASK | SP_PAPER_SIZE_LETTER_MASK)
#define SP_PAPER_SIZE_LEGAL SP_PAPER_SIZE_LEGAL_MASK
#define SP_PAPER_SIZE_TABLOID SP_PAPER_SIZE_TABLOID_MASK
#define SP_PAPER_SIZE_HAGAKI SP_PAPER_SIZE_HAGAKI_MASK
#define SP_PAPER_SIZE_STATEMENT SP_PAPER_SIZE_STATEMENT_MASK
#define SP_PAPER_SIZE_QUARTO SP_PAPER_SIZE_QUARTO_MASK
#define SP_PAPER_SIZE_FOOLSCAP SP_PAPER_SIZE_FOOLSCAP_MASK
#define SP_PAPER_SIZE_EXECUTIVE SP_PAPER_SIZE_EXECUTIVE_MASK
#define SP_PAPER_SIZE_CUSTOM SP_PAPER_SIZE_CUSTOM_MASK
    
#if defined(MACOS)
#pragma import on
#endif

extern void spsetstdio(FILE *in, FILE *out, FILE *err);
extern FILE *spgetstdin(void);
extern FILE *spgetstdout(void);
extern FILE *spgetstderr(void);
extern void spSetPrintFunc(spPrintFunc func);
extern void spPrint(const char *message, FILE *fp);

/** @~english @name Message Display
 *  @{ */

/** @~english Sets a flag to determine whether warning messages are displayed. */
extern void spSetWarningFlag(int flag);
/** @~english Displays a warning message to stderr. */
extern void spWarning(const char *format, ...);
/** @~english Sets a flag to determine whether normal messages are displayed. */
extern void spSetMessageFlag(int flag);
/** @~english Displays a normal message to stderr. */
extern void spMessage(const char *format, ...);
/** @~english Displays an error message to stderr, and then terminates the current program. */
extern void spError(int status, const char *format, ...);
/** @~english Displays an error message and the name of the function causing the error to stderr, and then terminates the current program. */
extern void spProgError(const char *func_name, const char *format, ...);
/** @~english Sets a flag to determine whether the desitination of debug messages is stdout or stderr. */
extern void spSetDebugStdout(int flag);
/** @~english Sets a debug level. */
extern void spSetDebugLevel(int level);
/** @~english Gets the current debug level. */
extern int spGetDebugLevel(void);
/** @~english Displays a debug message. */
extern void spDebug(int level, const char *func_name, const char *format, ...);

/** @~english @} */

/** @~english @name System Control
 *  @{ */
 
/** @~english Suspends the execution of the current program for the specified interval in seconds. */
extern void spSleep(int sec);
/** @~english Suspends the execution of the current program for the specified interval in milliseconds. */
extern void spMSleep(int msec);
/** @~english Gets the unique ID for the current process. */
extern long spGetProcessId(void);

/** @~english Opens the URL with a web browser.
 *  @param[in] base_url The main part of URL including scheme, host, etc. This can be a local path (file or directory).
 In this case, \p local_path must be SP_TRUE.
 *  @param[in] sub_url The optional path following the main part. This part must be used if base_url is a local path
 and the URL includes '\#' such as 'index.html\#foo'.
 *  @param[in] local_path SP_TRUE if \p base_url is a local path.
 *  @retval SP_TRUE Success  @retval SP_FALSE Failure  */
extern spBool spExecBrowser(const char *base_url, const char *sub_url, spBool local_path);

/** @~english @} */

/** @~english @name String Management
 *  @{ */

/** @~english Compares the two strings ignoring the case of the characters. */
extern int spStrCaseCmp(const char *s1, const char *s2);
/** @~english Compares the two strings ignoring the case of the characters. */
extern int spStrNCaseCmp(const char *s1, const char *s2, int n);
/** @~english Checks whether the string consists of white characters only. */
extern int spStrWhite(const char *string);
/** @~english Skips white characters in the string. */
extern char *spSkipWhite(const char *string);
/** @~english Concatenates two strings. */
extern char *xspStrCat(const char *orig, const char *src);
/** @~english Concatenates two strings. */
extern char *spStrCat(char *dest, int destsize, const char *src); /* dest is always terminated with '\0' regardless of src's length */
/** @~english Copies the contents of a string to another. */
extern char *spStrCopy(char *dest, int destsize, const char *src); /* dest is always terminated with '\0' regardless of src's length */
/** @~english Searches the suffix from the string. */
extern char *spGetSuffix(const char *name); /* including dot such as ".wav" */
/** @~english Checks whether two strings have the same suffix. */
extern int spEqSuffix(const char *file1, const char *file2);
/** @~english Removes the suffix in the string. */
extern spBool spRemoveNSuffix(char *path, char *removed_suffix, int suffixsize);
/** @~english Removes the suffix in the string. */
extern spBool spRemoveSuffix(char *path, char *removed_suffix);
/** @~english Replaces a suffix of the string with another. If the string doesn't have a suffix, a new suffix will be appended. */
extern spBool spReplaceNSuffix(char *path, int pathsize, const char *suffix);
/** @~english Replaces a suffix of the string with another. If the string doesn't have a suffix, a new suffix will be appended. */
extern spBool spReplaceSuffix(char *path, const char *suffix);
    
/** @~english Prepends the escape character '\\' to a character specified by \p charlist .
 *  @param[in] escaped The output null-terminated string.
 *  @param[in] bufsize The buffer size of \p escaped .
 *  @param[in] string The input null-terminated string.
 *  @param[in] charlist The list of the target characters.
 *  @return The number of insertions of '\\' for success, or -1 for failure. */
extern int spConvertToEscapedString(char *escaped, int bufsize, const char *string, const char *charlist);
/* return:  */
/** @~english Deletes the escape character '\\' before a character specified by \p charlist from the input string.
 *  @param[in] string The output null-terminated string.
 *  @param[in] bufsize The buffer size of \p string .
 *  @param[in] escaped The input null-terminated string including escape characters.
 *  @param[in] charlist The list of the target characters. If \p charlist is NULL, all escapes by '\\' will be deleted.
 *  @return The number of deletions of '\\' for success, or -1 for failure.   */
extern int spConvertFromEscapedString(char *string, int bufsize, const char *escaped, const char *charlist);
    
/** @~english @} */

/** @internal */
extern void spStrPToC(const unsigned char *in_p, char *out_c);
extern void spStrCToP(const char *in_c, unsigned char *out_p);
extern void spCopyPStr(const unsigned char *source, unsigned char *dest);
extern void spCatPStr(unsigned char *target, const unsigned char *source);
extern spBool spGetMacFileInfo(const char *filename, unsigned long *creator, unsigned long *file_type); /* application info if filename == NULL */
extern spBool spSetMacFileInfo(const char *filename, unsigned long creator, unsigned long file_type);

/** @~english @name File and Directory
 *  @{ */

/** @~english Opens the file whose name is \p path . */
extern FILE *spOpenFile(const char *path, const char *mode);
/** @~english Closes the stream opened by spOpenFile() . */
extern int spCloseFile(FILE *stream);
/** @~english Moves the file pointer to the specified location. */
extern int spSeekFile(FILE *stream, spFileOffset offset, int origin);
#ifdef SP_SUPPORT_64BIT_INT
/** @~english Moves the file pointer to the specified location. */
extern int spSeekFile64(FILE *stream, spLong64 offset, int origin);
#endif
/** @internal */
extern int spSeekFileC64(FILE *stream, char offset_c64[8], int swap_endian, int origin);
/** @internal */
extern void spAddC64(char offset_c64[8], long value, int swap_endian);
/** @internal */
extern spULong spC64ToULong(char value_c64[8], int swap_endian);
/** @internal */
extern void spULongToC64(spULong value, char value_c64[8], int swap_endian);
#ifdef SP_SUPPORT_64BIT_INT
/** @internal */
extern spLong64 spC64ToLong64(char value_c64[8], int swap_endian);
/** @internal */
extern void spLong64ToC64(spLong64 value, char value_c64[8], int swap_endian);
#endif
/** @~english Gets the current position of the file pointer. */
extern spFileOffset spTellFile(FILE *stream);
/** @~english Gets the information of the path. */
extern spBool spGetFileInfo(const char *path, spBool *dir_flag, spFileOffset *size);
/** @~english Checks whether the file or directory exists. */
extern spBool spExists(const char *path);
/** @~english Checks whether the file or directory exists. */
extern spBool spIsExist(const char *path);
/** @~english Checks whether the path represents a file. */
extern spBool spIsFile(const char *path);
/** @~english Checks whether the path represents a directory. */
extern spBool spIsDir(const char *path);
/** @~english Creates a directory. */
extern spBool spCreateDir(const char *path, unsigned short mode);
/** @~english Creates a directory recursively. */
extern spBool spCreateDirRecursive(const char *path, unsigned short mode);
/** @~english Removes a directory. */
extern spBool spRemoveDir(const char *path);
/** @~english Removes a file. */
extern spBool spRemoveFile(const char *path);
/** @~english Renames a file or directory. */
extern spBool spRenameFile(const char *oldpath, const char *newpath);

/** @~english @} */

/** @~english @name File and Directory Names
 *  @{ */
 
/** @~english Adds a character for directory separator to the path. */
extern spBool spAddDirSeparator(char *dir);
/** @~english Removes a character for directory separator from the path. */
extern spBool spRemoveDirSeparator(char *dir);
/** @~english Checks whether the name is an exact path name. */
extern spBool spIsExactName(const char *name);
/** @~english Gets the exact path. */
extern char *xspGetExactName(const char *name);
/** @~english Strips a directory path from the specified path. Note that the argument \p name is not modified by this function call. */
extern char *spGetBaseName(const char *name);
/** @~english Strips a directory path from the specified path. */
extern char *xspGetBaseName(const char *name);
/** @~english Extract a directory path from the specified path. Note that the argument \p filename is modified by this function call. */
extern char *spGetDirName(char *filename);
/** @~english Extract a directory path from the specified path. */
extern char *xspGetDirName(const char *filename);
/** @internal */
extern spBool spSetMacReadableEncoding(unsigned int encoding);
/** @~english Gets a path for the use of display from the real path. */
extern char *xspGetReadablePath(const char *real_path);
/** @~english Gets a real path from the path for the use of display. */
extern char *xspGetRealPath(const char *readable_path);

/** @~english Extracts a path from the list of paths. */
extern char *xspCutPathList(const char *pathlist, int col);
/** @~english Appends a new path list to the tail of the current path list. */
extern char *xspAppendPathList(const char *pathlist, const char *new_list);
/** @~english Prepends a new path list to the head of the current path list. */
extern char *xspPrependPathList(const char *pathlist, const char *new_list);

/** @~english @} */

/** @~english @name System and Application Directory
 *  @{ */
 
/** @~english Gets the current directory. */
extern const char *spGetCurrentDir(void);
/** @~english Gets the current directory. */
extern char *xspGetCurrentDir(void);
extern const char *spGetDefaultDir(void);
/** @~english Gets the home directory. */
extern const char *spGetHomeDir(void);
/** @~english Gets the home directory. */
extern char *xspGetHomeDir(void);
/** @~english Gets the temporal directory. */
extern const char *spGetTempDir(void);
/** @~english Gets the temporal directory. */
extern char *xspGetTempDir(void);

/** @~english Sets the application ID. This ID can includes the company ID and the version ID by using the character '/' as a separator. */
extern spBool spSetApplicationId(const char *id);
/** @~english Gets the company ID from the ID specified by spSetApplicationId() . */
extern const char *spGetCompanyId(void);
/** @~english Gets the application ID from the ID specified by spSetApplicationId() . */
extern const char *spGetApplicationId(void);
/** @~english Gets the version ID from the ID specified by spSetApplicationId() . */
extern const char *spGetVersionId(void);
extern const char *spGetApplicationLibDir(void);
extern const char *spGetApplicationDocDir(void);
    
extern spBool spGetApplicationPath(char *application_path, int path_size, spBool *id_depend,
				   const char *application_id, const char *company_id, spBool create_dir);
extern spBool spGetVersionApplicationPath(char *version_application_path, int path_size, 
					  const char *application_path, const char *version_id, spBool create_dir);
extern spBool spGetApplicationTempPath(char *application_temp_path, int path_size,
				       const char *application_path, const char *application_id, const char *version_id,
				       spBool id_depend, spBool create_dir);
    
/** @~english Creates the application directory which corresponds to the ID specified by spSetApplicationId() . */
extern const char *spCreateApplicationDir(spBool *id_depend, spBool *version_depend);
/** @~english Gets the application directory. If spCreateApplicationDir() has not been called yet, this function creates a new application directory. */
extern const char *spGetApplicationDir(spBool *version_depend);
/** @~english Gets the application directory. If spCreateApplicationDir() has not been called yet, this function creates a new application directory. */
extern char *xspGetApplicationDir(spBool *version_depend);
/** @~english Gets the temporal directory for the current application. */
extern const char *spGetApplicationTempDir(void);
/** @~english Gets the temporal directory for the current application. */
extern char *xspGetApplicationTempDir(void);
/** @~english Sets the temporal directory for the current application. */
extern spBool spSetApplicationTempDir(const char *dir);
/** @~english Gets the special directory for a kind of files. */
extern char *xspGetDocumentDir(spFileKind file_kind);

/** @~english @} */

/** @~english @name Numeric Functions
 *  @{ */

/** @~english Rounds to the nearest integer. */
extern double spRound(double x);
/** @~english Rounds toward zero. */
extern double spFix(double x);
/** @~english Gets the fractional part of \p x with the same sign. */
extern double spFrac(double x);
/** @~english Calculates remainder after division. */
extern double spRem(double x, double y);
/** @~english Calculates the factorial. */
extern long spFactorial(int n);
/** @~english Converts the floating point value to a null-terminated string. */
extern void spNFtos(char *buf, int bufsize, double x);
/** @~english Converts the floating point value to a null-terminated string. */
extern void spFtos(char *buf, double x);
/** @~english Calculates the exponent for the next power of 2. */
extern long spNextPow2(long n);

/** @~english @} */

/** @~english @name Type Conversion
 *  @{ */
 
/** @~english Gets a size in bytes for the specified bit format. */
extern int spGetBitByte(int samp_bit, spBool *float_flag);
/** @~english Gets a weighting factor to normalize the specified bit format. */
extern double spGetBitNormalizeFactor(int samp_bit);
/** @~english Converts a buffer of double type into a buffer of the specified bit format. */
extern long spConvertDoubleToBitWeighted(double *idata, long length, void *odata, int samp_bit, double weight);
/** @~english Converts a buffer of double type into a buffer of the specified bit format. */
extern long spConvertDoubleToBit(double *idata, long length, void *odata, int samp_bit, double max);
/** @~english Converts a buffer of double type into a buffer of the specified bit format with hard clipping. */
extern long spConvertDoubleToClippedBitWeighted(double *idata, long length, void *odata, int samp_bit, double weight);
/** @~english Converts a buffer of double type into a buffer of the specified bit format with hard clipping. */
extern long spConvertDoubleToClippedBit(double *idata, long length, void *odata, int samp_bit, double max);
/** @~english Converts a buffer of the specified bit format into a buffer of double type. */
extern long spConvertBitToDoubleWeighted(void *idata, long length, double *odata, int samp_bit, double weight);
/** @~english Converts a buffer of the specified bit format into a buffer of double type. */
extern long spConvertBitToDouble(void *idata, long length, double *odata, int samp_bit, double max);

/** @~english @} */

/** @internal */
extern spBool spGetPaperDimensions(spPaperSize paper_size, spPaperOrientation paper_orientation,
				   double *width_mm, double *height_mm);
extern spBool spGetPaperSizeString(spPaperSize paper_size, unsigned long options, char *buf, int buf_size);
extern spBool spGetPaperDimensionString(spPaperSize paper_size, unsigned long options, char *buf, int buf_size);
extern spBool spGetPaperFromDimensions(double width_mm, double height_mm,
				       spPaperSize *paper_size, spPaperOrientation *paper_orientation);
    
#if defined(MACOS)
extern spBool spIsAltiVecSupportedMac(void);
extern spBool spIsAltiVecDoubleSupportedMac(void);
#endif
    
#if defined(MACOS) && !defined(MACOSX64)
extern spBool spFSSpecToExactNameMac(/*FSSpec *fsspec*/void *fsspec, char *filename);

#pragma import off
#endif

#define spStrCaseEq(s1, s2) ((s1 != NULL) && (s2 != NULL) && (spStrCaseCmp((s1), (s2)) == 0) ? 1 : 0)
#define spStrNCaseEq(s1, s2, n) ((s1 != NULL) && (s2 != NULL) && (spStrNCaseCmp((s1), (s2), n) == 0) ? 1 : 0)
#define spStrVCaseEq(s1, s2) ((s1 != NULL) && (s2 != NULL) && (spStrNCaseCmp((s1), (s2), (int)strlen(s2)) == 0) ? 1 : 0)

/** @} */  /*----@addtogroup spBaseGroup----*/

#if !defined(SP_DISABLE_ABBREVIATION)
#ifndef strcaseeq
#define strcaseeq spStrCaseEq
#endif
#ifndef strncaseeq
#define strncaseeq spStrNCaseEq
#endif
#ifndef strvcaseeq
#define strvcaseeq spStrVCaseEq
#endif

#ifndef strwhite
#define strwhite spStrWhite
#endif

#ifndef xstrcat
#define xstrcat xspStrCat
#endif
#endif /* !defined(SP_DISABLE_ABBREVIATION) */

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPBASE_H */
