/** @file sp/spDefs.h
 */

#ifndef __SPDEFS_H
#define __SPDEFS_H

#if defined(_WIN32)
#define _USE_MATH_DEFINES 
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#if defined(_WIN32) && defined(_DEBUG) && defined(_CRTDBG_MAP_ALLOC)
/*#define _CRTDBG_MAP_ALLOC*/
#include <crtdbg.h>
#endif

#if defined(ANDROID)
#include <android/log.h>
#include <endian.h>
#elif defined(__APPLE__) && defined(__MACH__)
#include <TargetConditionals.h>
#include <AvailabilityMacros.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spDefsGroup Basic Definitions */

/** @addtogroup spDefsGroup
 *  @{ */  /*----@addtogroup spDefsGroup----*/

#define SPB_VERSION_STRING	"0.8.24"
#define SPB_VERSION 		0
#define SPB_REVISION		8
#define SPB_UPDATE_LEVEL	24
#define SPB_VERSION_ID		(SPB_VERSION * 1000 + SPB_REVISION)

/** @~english Checks if the version of the spBase header files is the same as or newer than the specified version.
 *  @param[in] version The version number.
 *  @param[in] revision The revision number.
 *  @param[in] update The update number.
 */
#define SPB_CHECK_VERSION(version, revision, update) \
    (SPB_VERSION > (version) || \
     (SPB_VERSION == (version) && SPB_REVISION > (revision)) || \
     (SPB_VERSION == (version) && SPB_REVISION == (revision) && SPB_UPDATE_LEVEL >= (update)))
    
typedef int (*spMainFunc)(int argc, char *argv[]);

#if defined(__CYGWIN__)
#if !defined(_WIN32_IE)
#define _WIN32_IE 0x0400
#endif
#if !defined(_WIN32)
#define _WIN32
#endif
#if !defined(__CYGWIN32__)
#define __CYGWIN32__
#endif
#endif

#if (TARGET_RT_MAC_MACHO || defined(__APPLE__)) && !defined(MACOSX)
#define MACOSX
#endif

#if defined(MACOSX)
#if !defined(IPHONE)
#if (defined(TARGET_OS_IPHONE) && TARGET_OS_IPHONE) || (defined(TARGET_IPHONE_SIMULATOR) && TARGET_IPHONE_SIMULATOR)
#define IPHONE 1
#endif
#endif /* !defined(IPHONE) */

#if !defined(IPHONE) && !defined(__LP64__) && !defined(COCOA)
#ifndef TARGET_API_MAC_CARBON
#define TARGET_API_MAC_CARBON 1
#endif
#endif /* !defined(IPHONE) && !defined(__LP64__) && !defined(COCOA) */
#ifndef MACOS
#define MACOS
#endif
#ifndef powerc
#define powerc 1
#endif
#if defined(__LP64__) && !defined(MACOSX64)
#define MACOSX64
#endif
#ifndef HAVE_PTHREAD
#define HAVE_PTHREAD
#endif
#endif /* defined(MACOSX) */

#ifndef MACOS
#if (defined(__MWERKS__) && defined(macintosh)) || defined(applec) || defined(__MRC__) ||\
    defined(THINK_C) || defined(__SC__) || (defined(TARGET_OS_MAC) && TARGET_OS_MAC == 1)
#define MACOS
#if !defined(powerc) && defined(__MRC__)
#define powerc 1
#endif
#endif
#endif
	
#if defined(MACOS) && !defined(MACOSX)
#define MACOS9
#ifndef TARGET_API_MAC_CARBON
#define TARGET_API_MAC_CARBON 0
#endif
#endif /* defined(MACOS) && !defined(MACOSX) */
    
#if TARGET_API_MAC_CARBON
#ifndef HIGH_LEVEL_EVENT
#define HIGH_LEVEL_EVENT
#endif
#endif

#if defined(__CYGWIN32__) || defined(__MINGW32__) || (!defined(_WIN32) && !defined(MACOS9)) || defined(ANDROID)
#define SP_SUPPORT_UTF8_STRING
#endif
    
#ifndef __P
#if __STDC__
#define __P(args) args
#else
#define __P(args) ()
#endif
#endif

/* spLong: 32bit on 32bit platform, 64bit on 64bit platform */
#if defined(_WIN64)
#if defined(__MINGW64__) || defined(__CYGWIN64__)
#define spLong long long
#define spULong unsigned long long
#else /* defined(__MINGW64__) || defined(__CYGWIN64__) */
#define spLong __int64
#define spULong unsigned __int64
#endif /* !(defined(__MINGW64__) || defined(__CYGWIN64__)) */
#else /* defined(_WIN64) */
#define spLong long
#define spULong unsigned long
#endif /* !defined(_WIN64) */

#if defined(_WIN32) && !(defined(__CYGWIN32__) || defined(__MINGW32__))
#define SP_SUPPORT_64BIT_INT
#define spLong64 __int64
#define spULong64 unsigned __int64
#elif defined(MACOS9)
#if TYPE_LONGLONG
#define SP_SUPPORT_64BIT_INT
#define spLong64 SInt64
#define spULong64 UInt64
#endif
#else /* defined(MACOS9) */
#define SP_SUPPORT_64BIT_INT
#define spLong64 long long
#define spULong64 unsigned long long
#endif /* !defined(MACOS9) */
    
#if defined(__alpha__) || defined(__alpha) || defined(__LP64__) || defined(__ia64__) || \
    defined(__s390x__) || defined(__ppc64__) || defined(__x86_64__)
#define spLong32 int
#define spULong32 unsigned int
#define SP_LONG32_IS_INT
#else
#define spLong32 long
#define spULong32 unsigned long
#define SP_LONG32_IS_LONG
#endif

/** @def spLONG
 *  Native type whose size is the same as that of LONG on Windows. */
/** @def spULONG
 *  Native type whose size is the same as that of ULONG on Windows. */
/** @def spDWORD
 *  Native type whose size is the same as that of DWORD on Windows. */
/** @def spINT8
 *  Native type whose size is the same as that of INT8 on Windows. */
/** @def spINT16
 *  Native type whose size is the same as that of INT16 on Windows. */
/** @def spBYTE
 *  Native type whose size is the same as that of BYTE on Windows. */
/** @def spWORD
 *  Native type whose size is the same as that of WORD on Windows. */

#if defined(_WIN32)
#define spLONG spLong32
#define spULONG spULong32
#else /* defined(_WIN32) */
/* internal format */
#define spLONG long
#define spULONG unsigned long
#endif /* !defined(_WIN32) */
#define spDWORD spULONG
#define spINT8 signed char
#define spINT16 short
#define spBYTE unsigned char
#define spWORD unsigned short

#if defined(_WIN32) && !defined(__CYGWIN32__)
#define spFileOffset __int64
#elif defined(HAVE_FSEEKO)
#define spFileOffset off_t
#else
#define spFileOffset long
#endif
    
#if defined(_WIN32) && !defined(__CYGWIN32__)
#if defined(_MSC_VER) && (_MSC_VER >= 1400/*VS2005*/)
#define HAVE_VSPRINTF_S 1
#define HAVE_SPRINTF_S 1
#define HAVE_FOPEN_S 1
#define HAVE_STRCPY_S 1
#define HAVE_STRCAT_S 1
#define HAVE_STRNCPY_S 1
#define HAVE_STRNCAT_S 1
#define HAVE_SPLITPATH_S 1
#define HAVE_DUPENV_S 1
#define HAVE_SSCANF_S 1
#endif
#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L/*C99*/)
#define HAVE_VSNPRINTF 1
#define HAVE_SNPRINTF 1
#endif

#if defined(sun) && !defined(HAVE_MEMCPY)
#define memmove(dst,src,len) bcopy((char *)(src),(char *)(dst),(int)(len))
#define memcpy(dst,src,len) bcopy((char *)(src),(char *)(dst),(int)(len))
#define memcmp(b1,b2,len) bcmp((char *)(b1),(char *)(b2),(int)(len))
#endif
	
#define SP_MAX_NAME	128
#define SP_MAX_LINE	192
#define SP_MAX_PATHNAME	256
#define SP_MAX_MESSAGE	1024
#define SP_MAX_IE_URL	2084
#define SP_MAX_SUFFIX_SIZE 8
#define SP_MAX_LANGUAGE 32	/* for locale */
#define SP_EPSILON	/*1.0e-10*/2.2204460492503131e-016
#define SP_TINY_NUMBER 1.0e-10
#define SP_EPSILONF	/*1.0e-10f*/1.192092896e-07f
#define SP_TINY_NUMBERF 1.0e-10f

/** Boolean for spLibs. */
typedef int spBool;

#define SP_TRUE  	1       /**< True for spBool. */
#define SP_FALSE 	0       /**< False for spBool. */
#define SP_TRUE_STRING	"1"     /**< String representation of #SP_TRUE . */
#define SP_FALSE_STRING	"0"     /**< String representation of #SP_FALSE . */

#define SP_ON  		SP_TRUE
#define SP_OFF 		SP_FALSE
#define SP_SUCCESS 	1       /**< Status indicating success. */
#define SP_FAILURE 	0       /**< Status indicating failure. */
#define SP_EXIT_SUCCESS 0
#define SP_EXIT_FAILURE 1

#define SP_UNIX_DEFAULT_PATH_LIST "/bin:/usr/bin:/usr/X11R6/bin:/usr/local/bin:~/bin"
#define SP_UNIX_DEFAULT_LIBRARY_PATH_LIST "/usr/lib:/usr/local/lib:~/lib"
#define SP_UNIX_DEFAULT_SHARE_PATH_LIST "/usr/share:/usr/local/share:~/share"
#define SP_BROWSER_COMMAND_ENV "SP_BROWSER"
    
#define SP_WIN_DIR_SEPARATOR '\\'
#define SP_WIN_DIR_SEPARATOR_STRING "\\"
#define SP_UNIX_DIR_SEPARATOR '/'
#define SP_UNIX_DIR_SEPARATOR_STRING "/"
#define SP_MAC_DIR_SEPARATOR ':'
#define SP_MAC_DIR_SEPARATOR_STRING ":"

#define SP_WIN_PARENT_DIR_STRING ".."
#define SP_WIN_CURRENT_DIR_STRING "."
#define SP_UNIX_PARENT_DIR_STRING ".."
#define SP_UNIX_CURRENT_DIR_STRING "."
#define SP_MAC_PARENT_DIR_STRING "::"
#define SP_MAC_CURRENT_DIR_STRING ":"

#define SP_WIN_SEPARATED_PARENT_DIR_STRING "..\\"
#define SP_WIN_SEPARATED_CURRENT_DIR_STRING ".\\"
#define SP_UNIX_SEPARATED_PARENT_DIR_STRING "../"
#define SP_UNIX_SEPARATED_CURRENT_DIR_STRING "./"
#define SP_MAC_SEPARATED_PARENT_DIR_STRING "::"
#define SP_MAC_SEPARATED_CURRENT_DIR_STRING ":"

/** @def SP_PATHLIST_SEPARATOR
 *  Separator for the path list string which is a string containing multiple paths.
 *  The separator is ';' on Windows and Mac, or ':' on others. */

#if defined(_WIN32)
#if !defined(__CYGWIN32__)
#define SP_DIR_SEPARATOR SP_WIN_DIR_SEPARATOR
#define SP_DIR_SEPARATOR_STRING SP_WIN_DIR_SEPARATOR_STRING
#define SP_PARENT_DIR_STRING SP_WIN_PARENT_DIR_STRING
#define SP_CURRENT_DIR_STRING SP_WIN_CURRENT_DIR_STRING
#define SP_SEPARATED_PARENT_DIR_STRING SP_WIN_SEPARATED_PARENT_DIR_STRING
#define SP_SEPARATED_CURRENT_DIR_STRING SP_WIN_SEPARATED_CURRENT_DIR_STRING
#define SP_ANOTHER_DIR_SEPARATOR SP_UNIX_DIR_SEPARATOR
#define SP_ANOTHER_DIR_SEPARATOR_STRING SP_UNIX_DIR_SEPARATOR_STRING
#else
#define SP_DIR_SEPARATOR SP_UNIX_DIR_SEPARATOR
#define SP_DIR_SEPARATOR_STRING SP_UNIX_DIR_SEPARATOR_STRING
#define SP_PARENT_DIR_STRING SP_UNIX_PARENT_DIR_STRING
#define SP_CURRENT_DIR_STRING SP_UNIX_CURRENT_DIR_STRING
#define SP_SEPARATED_PARENT_DIR_STRING SP_UNIX_SEPARATED_PARENT_DIR_STRING
#define SP_SEPARATED_CURRENT_DIR_STRING SP_UNIX_SEPARATED_CURRENT_DIR_STRING
#define SP_ANOTHER_DIR_SEPARATOR SP_WIN_DIR_SEPARATOR
#define SP_ANOTHER_DIR_SEPARATOR_STRING SP_WIN_DIR_SEPARATOR_STRING
#endif
    
#define SP_DEFAULT_DIRECTORY "c:"
#define SP_PATHLIST_SEPARATOR ';'

#define SP_DEFAULT_LF_STRING "\r\n"

#elif defined(MACOS9)

#define SP_DIR_SEPARATOR SP_MAC_DIR_SEPARATOR
#define SP_DIR_SEPARATOR_STRING SP_MAC_DIR_SEPARATOR_STRING
#define SP_PARENT_DIR_STRING SP_MAC_PARENT_DIR_STRING
#define SP_CURRENT_DIR_STRING SP_MAC_CURRENT_DIR_STRING
#define SP_SEPARATED_PARENT_DIR_STRING SP_MAC_SEPARATED_PARENT_DIR_STRING
#define SP_SEPARATED_CURRENT_DIR_STRING SP_MAC_SEPARATED_CURRENT_DIR_STRING
    
#define SP_ANOTHER_DIR_SEPARATOR SP_UNIX_DIR_SEPARATOR
#define SP_ANOTHER_DIR_SEPARATOR_STRING SP_UNIX_DIR_SEPARATOR_STRING
#define SP_DEFAULT_DIRECTORY ":"
#define SP_PATHLIST_SEPARATOR ';'

#if defined(MACOSX)
#define SP_DEFAULT_LF_STRING "\n"
#else
#define SP_DEFAULT_LF_STRING "\r"
#endif

#else

#define SP_DIR_SEPARATOR SP_UNIX_DIR_SEPARATOR
#define SP_DIR_SEPARATOR_STRING SP_UNIX_DIR_SEPARATOR_STRING
#define SP_PARENT_DIR_STRING SP_UNIX_PARENT_DIR_STRING
#define SP_CURRENT_DIR_STRING SP_UNIX_CURRENT_DIR_STRING
#define SP_SEPARATED_PARENT_DIR_STRING SP_UNIX_SEPARATED_PARENT_DIR_STRING
#define SP_SEPARATED_CURRENT_DIR_STRING SP_UNIX_SEPARATED_CURRENT_DIR_STRING

#if defined(MACOSX)
#define SP_ANOTHER_DIR_SEPARATOR SP_MAC_DIR_SEPARATOR
#define SP_ANOTHER_DIR_SEPARATOR_STRING SP_MAC_DIR_SEPARATOR_STRING
#define SP_DEFAULT_DIRECTORY "/"
#define SP_PATHLIST_SEPARATOR ';'
#else
#define SP_ANOTHER_DIR_SEPARATOR SP_UNIX_DIR_SEPARATOR
#define SP_ANOTHER_DIR_SEPARATOR_STRING SP_UNIX_DIR_SEPARATOR_STRING
#define SP_DEFAULT_DIRECTORY "/"
#define SP_PATHLIST_SEPARATOR ':'
#endif

#define SP_DEFAULT_LF_STRING "\n"
#endif

#define spMax(a, b) ((a) > (b) ? (a) : (b)) /**< The largest of \p a and \p b . */
#define spMin(a, b) ((a) < (b) ? (a) : (b)) /**< The smallest of \p a and \p b . */
#define spAbs(x) ((x) >= 0 ? (x) : -(x))    /**< The absolute value of \p x .  */
#define spCAbs(xr, xi) sqrt((double)(xr)*(double)(xr)+(double)(xi)*(double)(xi)) /**< The absolute value of the complex number.  */
#define spCAbsf(xr, xi) sqrtf((float)((xr)*(xr)+(xi)*(xi))) /**< The absolute value of the complex number for float type.  */
#define spSquare(x) ((x) * (x))                             /**< The square of \p x .  */
#define spCSquare(xr, xi) ((xr)*(xr)+(xi)*(xi))             /**< The absolute square of the complex number.   */
#define spPow2(p) (1 << (long)(p))                          /**< 2 to the \p p th power. */

#define spArraySize(array) ((unsigned int)(sizeof(array) / sizeof(array[0]))) /**< Gets size of the array. */
#define spStrEq(s1, s2) ((s1 != NULL) && (s2 != NULL) && (strcmp((s1), (s2)) == 0) ? 1 : 0) /**< True if two strings are the same. */
#define spStrNEq(s1, s2, n) ((s1 != NULL) && (s2 != NULL) && (strncmp((s1), (s2), n) == 0) ? 1 : 0) /**< True if two strings until \p n the element are the same. */
#define spStrVEq(s1, s2) ((s1 != NULL) && (s2 != NULL) && (strncmp((s1), (s2), strlen(s2)) == 0) ? 1 : 0) /**< True if two strings until the length of \p s2 are the same. */
#define spStrNone(string) (((string) == NULL || *(string) == NUL) ? 1 : 0) /**< The string has no data. */
#define spIsTrue(flag) ((flag) != SP_FALSE ? SP_TRUE : SP_FALSE) /**< True if \p flag is not 0. */
#define spIsFalse(flag) ((flag) == SP_FALSE ? SP_TRUE : SP_FALSE) /**< True if \p flag is 0. */

#define spdBf(x) (20.0f * log10f((float)((x) <= 0.0f ? SP_TINY_NUMBER : (x)))) /**< Decibel of amplitude \p x for float type.  */
#define spdB(x) (20.0 * log10((double)((x) <= 0.0 ? SP_TINY_NUMBER : (x)))) /**< Decibel of amplitude \p x . */
#define spdBpowf(x) (10.0f * log10f((float)((x) <= 0.0f ? SP_TINY_NUMBER : (x)))) /**< Decibel of power \p x for float type. */
#define spdBpow(x) (10.0 * log10((double)((x) <= 0.0 ? SP_TINY_NUMBER : (x)))) /**< Decibel of power \p x. */

#ifdef XtOffset
#define spOffset XtOffset
#else
#define spOffset(typep, field) \
    ((int) (((char *) (&(((typep)0)->field))) - ((char *)0))) /**< Returns the offset of the specified field member within the pointer to struct. */
#endif

#ifdef XtOffsetOf
#define spOffsetOf XtOffsetOf
#else
#ifdef offsetof
#define spOffsetOf offsetof
#else
#define spOffsetOf(type, field) spOffset(type*, field) /**< Returns the offset of the specified field member within the struct. */
#endif
#endif
    
#define SP_NUL		'\0'    /**< The null character. */

#if !defined(SP_DISABLE_ABBREVIATION)
#ifdef NUL
#undef NUL
#endif
#define NUL		SP_NUL

#ifndef NULL
#define NULL		0
#endif

#ifndef PI
#ifdef M_PI
#define PI		M_PI
#else
#define PI		3.1415926535897932385
#endif
#endif

#ifndef RAND_MAX
#ifdef INT_MAX
#define RAND_MAX INT_MAX
#else
#define RAND_MAX 2147483647
#endif
#endif

#ifndef MAX
#define MAX spMax
#endif
#ifndef MIN
#define MIN spMin
#endif
#ifndef ABS
#define ABS spAbs
#endif
#ifndef FABS
#define FABS spAbs
#endif
#ifndef CABS
#define CABS spCAbs
#endif
#ifndef CABSF
#define CABSF spCAbsf
#endif
#ifndef SQUARE
#define SQUARE spSquare
#endif
#ifndef CSQUARE
#define CSQUARE spCSquare
#endif
#ifndef POW2
#define POW2 spPow2
#endif

#ifndef arrsize
#define arrsize spArraySize
#endif
#ifndef streq
#define streq spStrEq
#endif
#ifndef strneq
#define strneq spStrNEq
#endif
#ifndef strveq
#define strveq spStrVEq
#endif
#ifndef strnone
#define strnone spStrNone
#endif

#define dBf spdBf
#define dB spdB
#define dBpowf spdBpowf
#define dBpow spdBpow
#endif /* !defined(SP_DISABLE_ABBREVIATION) */

#if defined(ANDROID)
#define  printf(...)  __android_log_print(ANDROID_LOG_INFO,"printf",__VA_ARGS__)
#define  fprintf(fp, ...) 	\
    if (fp == stdout) {		\
	__android_log_print(ANDROID_LOG_INFO,"printf",__VA_ARGS__);	\
    } else if (fp == stderr) {						\
	__android_log_print(ANDROID_LOG_WARN,"printf",__VA_ARGS__);	\
    } else {								\
	fprintf(fp, __VA_ARGS__);					\
    }
#endif

#if defined(__CUDACC__)
#if !defined(_WIN32) || (defined(_WIN32) && defined(_DEBUG))
#define spCudaSafeCall(err) __spCudaSafeCall(err, __FILE__, __LINE__)
#define spCudaCheckMsg(msg) __spCudaCheckMsg(msg, __FILE__, __LINE__)
#else
#define spCudaSafeCall(err) err
#define spCudaCheckMsg(msg) 
#endif
inline void __spCudaSafeCall(cudaError err, const char *file, const int line)
{
    if (cudaSuccess != err) {
	fprintf(stderr, "%s(%i) : CUDA Runtime API error : %s.\n", file, line, cudaGetErrorString(err));
        exit(-1);
    }
}
    
inline void __spCudaCheckMsg(const char *errorMessage, const char *file, const int line)
{
    cudaError_t err = cudaGetLastError();
    
    if (cudaSuccess != err) {
        fprintf(stderr, "%s(%i) : CUDA error : %s : %s.\n",
                file, line, errorMessage, cudaGetErrorString(err));
        exit(-1);
    }
}
#endif
    
/** @} */  /*----@addtogroup spDefsGroup----*/

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPDEFS_H */
