/** @file sp/spDirectory.h
 */

#ifndef __SPDIRECTORY_H
#define __SPDIRECTORY_H

#include <sp/spDefs.h>
#if !defined(MACOS9) && !(defined(_WIN32) && !defined(__CYGWIN32__))
#include <dirent.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if defined(MACOS9) || (defined(_WIN32) && !defined(__CYGWIN32__))
typedef struct _DIR DIR;

#define MAXNAMLEN 1024

struct dirent {
    char d_name[MAXNAMLEN];
    unsigned short d_namlen;
    long d_ino;
};
#endif

#if defined(MACOS)
#pragma import on
#endif

/** @~english @defgroup spDirectoryGroup Directory */

/** @addtogroup spDirectoryGroup
 *  @{ */  /*----@addtogroup spDirectoryGroup----*/

/** @~english Opens the directory specified by \p path.
 *  @param[in] path The path of a directory to open.
 *  @return The directory pointer on success, or NULL on failure. */
extern DIR *spOpenDir(const char *path);
/** @~english Reads data of the directory.
 *  @param[in] dirp The directory pointer obtained by spOpenDir() .
 *  @return The pointer to struct dirent whose members of `d_name` and `d_namlen` are the name of the file associated with the pointer
 *  and the length of the name respectively. This function normally reads directories of '.' (the current directory) and '..' (the parent directory).
 *  NULL will be returned if error occurs or all files have been read.
 *  @sa spOpenDir() */
extern struct dirent *spReadDir(DIR *dirp);
/** @~english Closes the directory.
 *  @param[in] dirp The directory pointer obtained by spOpenDir() .
 *  @retval 0 Success  @retval -1 Failure
 *  @sa spOpenDir() */
extern int spCloseDir(DIR *dirp);

/** @} */  /*----@addtogroup spDirectoryGroup----*/

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPDIRECTORY_H */
