/** @file sp/spString.h
 */

#ifndef __SPSTRING_H
#define __SPSTRING_H

#include <sp/spDefs.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spStringGroup String-Related Functions */

/** @addtogroup spStringGroup
 *  @{ */  /*----@addtogroup spStringGroup----*/

/** Definition of spConverter . */
typedef struct _spConverter *spConverter;

#define SP_TIME_FORMAT_MSEC_MASK (1<<0)
#define SP_TIME_FORMAT_SEC_MASK (1<<1)
#define SP_TIME_FORMAT_POINT_MASK (1<<2)
#define SP_TIME_FORMAT_FLOORED_MASK (1<<9)
#define SP_TIME_FORMAT_SEPARATED_MASK (1<<10)
#define SP_TIME_FORMAT_USER1_MASK (1<<12)
#define SP_TIME_FORMAT_USER2_MASK (1<<13)
#define SP_TIME_FORMAT_USER3_MASK (1<<14)
#define SP_TIME_FORMAT_USER4_MASK (1<<15)

typedef enum {
    SP_TIME_FORMAT_UNKNOWN = 0,
    SP_TIME_FORMAT_MSEC = SP_TIME_FORMAT_MSEC_MASK,
    SP_TIME_FORMAT_FLOORED_MSEC = (SP_TIME_FORMAT_FLOORED_MASK|SP_TIME_FORMAT_MSEC_MASK),
    
    SP_TIME_FORMAT_SEC = SP_TIME_FORMAT_SEC_MASK,
    SP_TIME_FORMAT_FLOORED_SEC = (SP_TIME_FORMAT_FLOORED_MASK|SP_TIME_FORMAT_SEC_MASK),
    /* hh:mm:ss.ssssss */    
    SP_TIME_FORMAT_SEPARATED_SEC = (SP_TIME_FORMAT_SEPARATED_MASK|SP_TIME_FORMAT_SEC_MASK),
    /* hh:mm:ss */    
    SP_TIME_FORMAT_FLOORED_SEPARATED_SEC = (SP_TIME_FORMAT_FLOORED_MASK|SP_TIME_FORMAT_SEPARATED_MASK|SP_TIME_FORMAT_SEC_MASK),
    
    SP_TIME_FORMAT_POINT = SP_TIME_FORMAT_POINT_MASK,

    SP_TIME_FORMAT_USER1 = SP_TIME_FORMAT_USER1_MASK,
    SP_TIME_FORMAT_USER2 = SP_TIME_FORMAT_USER2_MASK,
    SP_TIME_FORMAT_USER3 = SP_TIME_FORMAT_USER3_MASK,
    SP_TIME_FORMAT_USER4 = SP_TIME_FORMAT_USER4_MASK,
} spTimeFormat;


#if defined(MACOS)
#pragma import on
#endif

/** @~english Checks whether two languages (locales) are the same. The language corresponds to a null-terminated string such as `"ja_JP.utf8"`. */
extern spBool spEqLanguage(const char *lang1, const char *lang2);
/** @~english Gets the language ID from the language (locale). For example, the language ID is `"ja"` for the language of `"ja_JP.utf8"`. */
extern spBool spGetLanguageId(const char *lang, char *id, int id_size);
/** @~english Checks whether the language (locale) is based on UTF. */
extern spBool spIsUTFLang(const char *lang);
/** @~english Checks whether the language (locale) is Japanese. */
extern spBool spIsJapaneseLang(const char *lang);
extern spBool spIsUTF8First(int c, int *continuation); /* continuation: continuation byte length */
extern spBool spIsMBTailCandidate(int prev_c, int c);
/** @~english Searches the position of the character \p c in the null-terminated string \p str . */
extern char *spStrChr(const char *str, int c);
/** @~english Searches the position of the character \p c in the null-terminated string \p str from the end of the string. */
extern char *spStrRChr(const char *str, int c);

extern int spUnicode16ToUTF8(const unsigned short *unicode, unsigned char *buf, int buf_size);
extern int spUTF8ToUnicode16(const unsigned char *utf8, unsigned short *buf, int buf_size);
extern int spUnicode16Length(const unsigned short *unicode);
extern unsigned short *spUnicode16Copy(unsigned short *dest, int destlen, const unsigned short *src);
extern unsigned short *xspUnicode16Clone(const unsigned short *unicode);
/** @~english Gets an internally valid encode name from the encode name. */
extern const char *spGetSupportedEncoding(const char *code);
/** @~english Opens a new converter for the specified input/output encode names. */
extern spConverter spOpenConverter(const char *icode, const char *ocode);
/** @~english Converts the encode of the input string into the another encode. */
extern char *xspConvert(spConverter conv, const char *input);
/** @~english Closes the converter. */
extern void spCloseConverter(spConverter conv);
    
extern spBool spGetTimeNString(double sec, spTimeFormat time_format, char *buf, int bufsize);
extern spBool spGetTimeString(double sec, spTimeFormat time_format, char *buf);
extern spTimeFormat spConvertTimeString(char *buf, spTimeFormat time_format, double *sec);
extern spBool spGetTimeFormatString(spTimeFormat time_format, char *format);
extern spBool spGetTimeFormatNString(spTimeFormat time_format, char *format, int formatsize);
extern spBool spConvertTimeFormatString(const char *format, spTimeFormat *time_format);

#if defined(MACOS)
#pragma import off
#endif

/** @} */  /*----@addtogroup spStringGroup----*/

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPSTRING_H */
